unit Utils;

interface

uses
  Windows, Classes, SysUtils, ShellApi;

{ תUTF8Ϊַ }
function UTF8ToWideString(S: AnsiString): WideString;

{ תַΪUTF8 }
function WideStringToUTF8(S: WideString): AnsiString;

{ ·β"\" }
function AddBackSlash(const S: string): string;

{ ȥ·β"\" }
function RemBackSlash(const S: string): string;

{ ȡϵͳʱĿ¼ }
function GetTempDir: string;

{ ɾָļ }
function Deltree(S: string): Boolean;

{ ȡָ·ļ }
procedure GetAllFiles(Path: string; const Filter: string; Dest: TStrings);

{ ȡļ }
function GetFileDate(const FileName: string): TDateTime;

{ ȡļС }
function FileSize(const FileName: string): LongInt;

{ ɾָļĿ¼ }
function DeleteFiles(const FileName: string; const fUseRecyleBin: Boolean = False; const fConfirm: Boolean = False): Boolean;

{ ַCRCֵ }
function CalculateCRC(s: string): Cardinal;

{ ָSeparatorsַָContent }
function SplitString(Separators: TSysCharSet; Content: string; Strings: TStrings): Integer;

implementation

uses
  CRC32;

type
  UCS4 = Cardinal;

const
  SurrogateHighStart: UCS4 = $D800;
  SurrogateHighEnd: UCS4 = $DBFF;
  SurrogateLowStart: UCS4 = $DC00;
  SurrogateLowEnd: UCS4 = $DFFF;
  halfShift: Integer = 10;
  halfBase: UCS4 = $0010000;
  halfMask: UCS4 = $3FF;
  ReplacementCharacter: UCS4 = $0000FFFD;
  MaximumUCS2: UCS4 = $0000FFFF;
  //MaximumUTF16: UCS4 = $0010FFFF;
  MaximumUCS4: UCS4 = $7FFFFFFF;
  offsetsFromUTF8: array[0..5] of UCS4 = ($00000000, $00003080, $000E2080, $03C82080, $FA082080, $82082080);
  bytesFromUTF8: array[0..255] of Byte = (
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2, 3,3,3,3,3,3,3,3,4,4,4,4,5,5,5,5);
  firstByteMark: array[0..6] of Byte = ($00, $00, $C0, $E0, $F0, $F8, $FC);

function UTF8ToWideString(S: AnsiString): WideString;
var
  L, J, T: Cardinal;
  ch: UCS4;
  extraBytesToWrite: Word;
begin
  if Length(S) = 0 then
  begin
    Result := '';
    Exit;
  end;

  SetLength(Result, Length(S)); // create enough room

  L := 1;
  T := 1;
  while L <= Cardinal(Length(S)) do
  begin
    ch := 0;
    extraBytesToWrite := bytesFromUTF8[Ord(S[L])];

    for J := extraBytesToWrite downto 1 do
    begin
      ch := ch + Ord(S[L]);
      Inc(L);
      ch := ch shl 6;
    end;
    ch := ch + Ord(S[L]);
    Inc(L);
    ch := ch - offsetsFromUTF8[extraBytesToWrite];

    if ch <= MaximumUCS2 then
    begin
      Result[T] := WideChar(ch);
      Inc(T);
    end
    else
    if ch > MaximumUCS4 then
    begin
      Result[T] := WideChar(ReplacementCharacter);
      Inc(T);
    end
    else
    begin
      ch := ch - halfBase;
      Result[T] := WideChar((ch shr halfShift) + SurrogateHighStart);
      Inc(T);
      Result[T] := WideChar((ch and halfMask) + SurrogateLowStart);
      Inc(T);
    end;
  end;
  SetLength(Result, T - 1); // now fix up length
end;

function WideStringToUTF8(S: WideString): AnsiString;
var
  ch: UCS4;
  L, J, T,
  bytesToWrite: Word;
  byteMask: UCS4;
  byteMark: UCS4;
begin
  if Length(S) = 0 then
  begin
    Result := '';
    Exit;
  end;

  SetLength(Result, Length(S) * 6); // assume worst case
  T := 1;
  for J := 1 to Length(S) do
  begin
    byteMask := $BF;
    byteMark := $80;

    ch := UCS4(S[J]);

    if ch < $80 then
      bytesToWrite := 1
    else
    if ch < $800 then
      bytesToWrite := 2
    else
    if ch < $10000 then
      bytesToWrite := 3
    else
    if ch < $200000 then
      bytesToWrite := 4
    else
    if ch < $4000000 then
      bytesToWrite := 5
    else
    if ch <= MaximumUCS4 then
      bytesToWrite := 6
    else
    begin
      bytesToWrite := 2;
      ch := ReplacementCharacter;
    end;

    for L := bytesToWrite downto 2 do
    begin
      Result[T + L - 1] := Char((ch or byteMark) and byteMask);
      ch := ch shr 6;
    end;
    Result[T] := Char(ch or firstByteMark[bytesToWrite]);
    Inc(T, bytesToWrite);
  end;
  SetLength(Result, T - 1); // assume worst case
end;

function AddBackSlash(const S: string): string;
begin
  Result := S;
  if Length(Result) > 0 then
    if Result[Length(Result)] <> '\' then
      Result := Result + '\';
end;

function RemBackSlash(const S: string): string;
begin
  Result := S;
  if Length(Result) > 0 then
    if Result[Length(Result)] = '\' then
      Result := Copy(S, 1, Length(S) - 1);
end;

function GetTempDir: string;
var
  Len: Longword;
begin
  SetLength(Result, MAX_PATH);
  Len := GetTempPath(MAX_PATH, PChar(Result));
  SetLength(Result, Len);
  Result := AddBackSlash(Result);
end;

function Deltree(S: string): Boolean;
var
  F: TSearchRec;
  Path, Wildcard: string;
  IsDir, Find: Boolean;
begin
  S := RemBackSlash(S);
  if ExtractFileName(S) = '' then //SǸĿ¼Ϊַ
  begin
    Result := False;
    Exit;
  end;
  Result := True;
  Find := False;
  IsDir := DirectoryExists(S);
  if IsDir then
  begin
    Path := ExpandFileName(S) + '\';
    Wildcard := Path + '*.*';
  end
  else
  begin
    Path := ExtractFilePath(S);
    Wildcard := S;
  end;
  if FindFirst(Wildcard, faAnyFile, F) = 0 then
  begin
    Find := True;
    while True do
    begin
      if not ((F.Name = '.') or (F.Name = '..')) then
      begin
        if (F.Attr and faDirectory) > 0 then
        begin
          if Deltree(Path + F.Name) = False then
            Result := False;
        end
        else
        begin
          {$WARNINGS OFF}
          FileSetAttr(Path + F.Name, 0);
          {$WARNINGS ON}
          if DeleteFile(Path + F.Name) = False then
            Result := False;
        end;
      end;
      if FindNext(F) <> 0 then
        Break;
    end;
    FindClose(F);
  end;
  if IsDir then
  begin
    if Result = True then
    begin
      {$WARNINGS OFF}
      FileSetAttr(S, faDirectory);
      {$WARNINGS ON}
      Result := RemoveDir(S);
    end;
  end
  else
    if not Find then
      Result := False;
end;

procedure GetAllFiles(Path: string; const Filter: string; Dest: TStrings);
var
  Rec: TSearchRec;
begin
  Dest.Clear;
  Path := AddBackSlash(Path);
  if FindFirst(Path + Filter, faAnyFile, Rec) <> 0 then
    Exit;
  repeat
    if FileExists(Path + Rec.Name) then
      Dest.Add(Rec.Name);
  until FindNext(Rec) <> 0;
  FindClose(Rec);
end;

function GetFileDate(const FileName: string): TDateTime;
begin
  if FileExists(FileName) then
    FileAge(FileName, Result)
  else
    Result := Now;
end;

function FileSize(const FileName: string): LongInt;
var
  FS: TFileStream;
  Mode: Word;
begin
  if not FileExists(FileName) then
  begin
    Result := 0;
    Exit;
  end;

  FS := nil;
  Mode := fmOpenRead or fmShareDenyNone;
  try
    FS := TFileStream.Create(FileName, Mode);
    Result := FS.Size;
  finally
    FS.Free;
  end;
end;

function DeleteFiles(const FileName: string; const fUseRecyleBin: Boolean = False; const fConfirm: Boolean = False): Boolean;
var
  APath: AnsiString;
  lpFileOp: TSHFileOpStruct;
begin
  APath := StringReplace(FileName, ',', #0, [rfReplaceAll]) + #0#0;
  FillChar(lpFileOp, SizeOf(lpFileOp), 0);
  with lpFileOp do
  begin
    Wnd := 0;
    wFunc := FO_DELETE;
    pFrom := PChar(APath);
    pTo := nil;
    fFlags := FOF_SILENT;
    if fUseRecyleBin then
      fFlags := fFlags + FOF_ALLOWUNDO;
    if not fConfirm then
      fFlags := fFlags + FOF_NOCONFIRMATION;
  end;
  Result := SHFileOperation(lpFileOp) = 0;
end;

function CalculateCRC(s: string): Cardinal;
begin
  Result := CalculateCRC32(s[1], Length(s));
end;

function SplitString(Separators: TSysCharSet; Content: string; Strings: TStrings): Integer;
begin
  Result := ExtractStrings(Separators, [' ', #9], PAnsiChar(Content), Strings);
end;

end.
